<?php
// $Id: drupalvb.inc.php,v 1.9.4.2 2008/04/22 19:09:33 sun Exp $

/**
 * @file
 * Drupal vB CRUD functions.
 */

/**
 * Set the necessary cookies for the user to be logged into the forum.
 */
function drupalvb_set_login_cookies($userid, $password, $salt) {
  $vb_config = drupalvb_get('config');
  $vb_options = drupalvb_get('options');

  $cookie_prefix = (empty($vb_config['Misc']['cookieprefix']) ? 'bb' : $vb_config['Misc']['cookieprefix']);
  $cookie_path = $vb_options['cookiepath'];
  $cookie_domain = $vb_options['cookiedomain'];
  $now = time();
  $expire = time() + 60 * 60 * 24 * 365;

  // Register user session.
  $sessionhash = md5('drupalvb'. $userid);
  $ip = implode('.', array_slice(explode('.', drupalvb_get_ip()), 0, 4 - $vb_options['ipcheck']));
  $idhash = md5($_SERVER['HTTP_USER_AGENT'] . $ip);
  drupalvb_db_query("INSERT IGNORE INTO {session} (sessionhash, userid, host, idhash, lastactivity, location, useragent) VALUES ('%s', %d, '%s', '%s', %d, '%s', '%s')", $sessionhash, $userid, $_SERVER['REMOTE_ADDR'], $idhash, $now, '/forum/', $_SERVER['HTTP_USER_AGENT']);

  setcookie($cookie_prefix .'sessionhash', $sessionhash, $expire, $cookie_path, $cookie_domain);
  setcookie($cookie_prefix .'lastvisit', $now, $expire, $cookie_path, $cookie_domain);
  setcookie($cookie_prefix .'lastactivity', $now, $expire, $cookie_path, $cookie_domain);
  setcookie($cookie_prefix .'userid', $userid, $expire, $cookie_path, $cookie_domain);
  setcookie($cookie_prefix .'password', md5($password . $salt), $expire, $cookie_path, $cookie_domain);
}

/**
 * Clear all vB cookies for the current user.
 */
function drupalvb_clear_cookies() {
  $vb_config = drupalvb_get('config');
  $vb_options = drupalvb_get('options');

  $cookie_prefix = (empty($vb_config['Misc']['cookieprefix']) ? 'bb' : $vb_config['Misc']['cookieprefix']);
  $cookie_path = $vb_options['cookiepath'];
  $cookie_domain = $vb_options['cookiedomain'];
  $expire = time() - 3600;
  
  // Frontend cookie names: lastvisit, lastactivity, sessionhash
  // Backend cookie names: cpsession, userid, password
  // However, in all cases cookiedomain is NOT prefixed with a dot unless cookie domain
  // has not been manually altered to either a suggested value or custom value in
  // vB's settings.
  setcookie($cookie_prefix .'sessionhash', '', $expire, $cookie_path, $cookie_domain);
  setcookie($cookie_prefix .'lastvisit', '', $expire, $cookie_path, $cookie_domain);
  setcookie($cookie_prefix .'lastactivity', '', $expire, $cookie_path, $cookie_domain);
  setcookie($cookie_prefix .'userid', '', $expire, $cookie_path, $cookie_domain);
  setcookie($cookie_prefix .'password', '', $expire, $cookie_path, $cookie_domain);
}

/**
 * Determines the IP address of current user.
 */
function drupalvb_get_ip() {
  $alt_ip = $_SERVER['REMOTE_ADDR'];

  if (isset($_SERVER['HTTP_CLIENT_IP'])) {
    $alt_ip = $_SERVER['HTTP_CLIENT_IP'];
  }
  else if (isset($_SERVER['HTTP_X_FORWARDED_FOR']) && preg_match_all('#\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}#s', $_SERVER['HTTP_X_FORWARDED_FOR'], $matches)) {
    // Make sure we dont pick up an internal IP defined by RFC1918
    foreach ($matches[0] AS $ip) {
      if (!preg_match("#^(10|172\.16|192\.168)\.#", $ip)) {
        $alt_ip = $ip;
        break;
      }
    }
  }
  else if (isset($_SERVER['HTTP_FROM'])) {
    $alt_ip = $_SERVER['HTTP_FROM'];
  }
  return $alt_ip;
}

/**
 * Create an entry in the vB database user table.
 */
function drupalvb_create_user($username, $password, $email, $silent = FALSE) {
  // Double check to make sure we're not duplicating an entry.
  $result = drupalvb_db_query("SELECT userid FROM {user} WHERE username = '%s'", $username);

  if (db_fetch_array($result)) {
    // The user existed somehow, so send the error and kick it out with a fail.
    // Skip pre-existing users without a message when exporting users from
    // Drupal to vBulletin.
    if (!$silent) {
      drupal_set_message(t('Your username already existed on the forums. The duplication problem has been logged. If this error message continues to display, please contact the site administrator.'), 'error');
      watchdog('drupalvb', t('Attempt to create a duplicate forum user.'), WATCHDOG_ERROR);
    }
    return false;
  }

  // Set up the necessary variables.
  $salt = '';
  for ($i = 0; $i < 3; $i++) {
    $salt .= chr(rand(32, 126));
  }

  // This hack expects users to not have 32 character length passwords.
  if (strlen($password) != 32) {
    $password = md5($password);
  }

  $passhash = md5($password . $salt);
  $passdate = date('Y-m-d H:i:s', time());
  $joindate = time();

  // Attempt to grab the user title from the database.
  $result = drupalvb_db_query("SELECT title FROM {usertitle} WHERE minposts = 0");

  if ($resarray = db_fetch_array($result)) {
    $usertitle = $resarray['title'];
  }
  else {
    $usertitle = 'Junior Member';
  }

  // Grab the default timezone offset, divided by 3600 b/c the system.module
  // timezone form is in seconds.
  $timezone = variable_get('drupalvb_default_timezone', '-18000');
  $timezone = intval($timezone) / 3600;

  // Default new user options: I got these by setting up a new user how I
  // wanted and looking in the database to see what options were set for him.
  $options = variable_get('drupalvb_default_options', '3415');

  // Default usergroup id.
  $usergroupid = variable_get('drupalvb_default_usergroup', '2');

  // Set up the insertion query.
  $result = drupalvb_db_query("INSERT INTO {user} (username, usergroupid, password, passworddate, usertitle, email, salt, showvbcode, languageid, timezoneoffset, posts, joindate, lastvisit, lastactivity, options) VALUES ('%s', '%s', '%s', '%s', '%s', '%s', '%s', 1, 1, '%s', 0, '%s', '%s', '%s', '%s')", $username, $usergroupid, $passhash, $passdate, $usertitle, $email, $salt, $timezone, $joindate, $joindate, $joindate, $options);

  $userid = mysql_insert_id();

  drupalvb_db_query("INSERT INTO {userfield} VALUES (%d, NULL, '', '', '', '')", $userid);
  drupalvb_db_query("INSERT INTO {usertextfield} VALUES (%d, NULL, NULL, NULL, NULL, '', NULL, NULL)", $userid);

  // Return true if the vB user was created successfully.
  return true;
}

/**
 * Update an entry in the vB database user table.
 */
function drupalvb_update_user($username, $password, $email, $salt) {
  // Set up the necessary variables.
  $lastdate = time();
  $setpass = '';

  if ($password != '') {
    // This hack expects users to not have 32 character length passwords.
    if (strlen($password) != 32) {
      $password = md5($password);
    }
    $passhash = md5($password . $salt);
    $passdate = date('Y-m-d H:i:s', time());
    drupalvb_db_query("UPDATE {user} SET password = '%s', passworddate = '%s', salt = '%s', email = '%s', lastactivity = '%s' WHERE username = '%s'", $passhash, $passdate, $salt, $email, $lastdate, $username);
  }
  else {
    drupalvb_db_query("UPDATE {user} SET email = '%s', lastactivity = '%s' WHERE username = '%s'", $email, $lastdate, $username);
  }
}

/**
 * Export all drupal users to vBulletin.
 */
function drupalvb_export_drupal_users() {
  require_once drupal_get_path('module', 'drupalvb') .'/drupalvb.inc';

  $result = db_query("SELECT uid FROM {users} ORDER BY uid");
  while ($data = db_fetch_object($result)) {
    if ($data->uid == 0) {
      continue;
    }
    $user = user_load(array('uid' => $data->uid));
    drupalvb_create_user($user->name, $user->pass, $user->mail, TRUE);
  }
}

/**
 * Get vBulletin configuration options.
 */
function drupalvb_get_options() {
  static $options = array();

  if (empty($options)) {
    $result = db_query("SELECT data FROM {datastore} WHERE title = 'options'");
    if ($data = db_fetch_array($result)) {
      $options = unserialize($data['data']);
    }
  }
  return $options;
}

/**
 * Get vBulletin configuration.
 */
function drupalvb_get_config() {
  static $config = array();

  // @todo Find & include vB's config automatically?
  // $files = file_scan_directory('.', '^config.php$', $nomask = array('.', '..', 'CVS', '.svn'));
  $config_file = drupal_get_path('module', 'drupalvb') .'/config.php';
  if (empty($config) && file_exists($config_file)) {
    require_once $config_file;
  }
  return $config;
}

/**
 * Get vB user roles.
 */
function drupalvb_get_roles() {
  $result = drupalvb_db_query("SELECT usergroupid, title FROM {usergroup}");

  $roles = array();
  while ($data = db_fetch_object($result)) {
    $roles[$data->usergroupid] = $data->title;
  }
  if (!$roles) {
    $roles[] = t('No user roles could be found.');
  }
  return $roles;
}

/**
 * Get counts of guests and members currently online.
 */
function drupalvb_get_users_online() {
  $vb_options = drupalvb_get('options');

  $datecut          = time() - $vb_options['cookietimeout'];
  $numbervisible    = 0;
  $numberregistered = 0;
  $numberguest      = 0;

  $result = drupalvb_db_query("SELECT user.username, user.usergroupid, session.userid, session.lastactivity FROM {session} AS session LEFT JOIN {user} AS user ON (user.userid = session.userid) WHERE session.lastactivity > %d", $datecut);

  $userinfos = array();

  while ($loggedin = db_fetch_array($result)) {
    $userid = $loggedin['userid'];
    if (!$userid) {
      $numberguest++;
    }
    else if (empty($userinfos[$userid]) || ($userinfos[$userid]['lastactivity'] < $loggedin['lastactivity'])) {
      $userinfos[$userid] = $loggedin;
    }
  }
  foreach ($userinfos as $userid => $loggedin) {
    $numberregistered++;
  }
  return array('guests' => $numberguest, 'members' => $numberregistered);
}

/**
 * Get counts of new or recent posts for the current user.
 */
function drupalvb_get_recent_posts($scope = 'last') {
  global $user;

  // Queries the vB user database to find a matching set of user data.
  $result = drupalvb_db_query("SELECT userid, username, lastvisit FROM {user} WHERE username = '%s'", $user->name);

  // Make sure a user is logged in to get their last visit and appropriate post
  // count.
  if ($vb_user = db_fetch_array($result)) {
    if ($scope == 'last') {
      $datecut = $vb_user['lastvisit'];
    }
    else if ($scope == 'daily') {
      $datecut = time() - 86400;
    }
    $result = drupalvb_db_query("SELECT postid FROM {post} WHERE dateline > %d", $datecut);
    $posts = db_num_rows($result);
  }
  else {
    $posts = 0;
  }
  return $posts;
}

